"""
.. index: Stanford Large Network Dataset Collection

.. index:
   single: Network; Stanford Large Network Dataset Collection
   
*****************************************
Stanford Large Network Dataset Collection
*****************************************

The classes in this module provide access to the
`Stanford Large Network Dataset Collection <http://snap.stanford.edu/data/>`_,
which is maintained by `Jure Leskovec <http://cs.stanford.edu/~jure/>`_.

.. autoclass:: Orange.network.snap.SNAP
   :members:

.. autoclass:: Orange.network.snap.NetworkInfo
   :members:

"""

import os
import urllib
import httplib

import Orange.misc

import readwrite

from HTMLParser import HTMLParser

class NetworkInfo(object):
    """The NetworkInfo class provides information about a network on the SNAP 
    web site.
    
    .. attribute:: name
    
        The name of the network.
    
    .. attribute:: link
    
        The url address of the network file.
    
    .. attribute:: type
    
        Network type (directed, undirected).
    
    .. attribute:: nodes
    
        Number of nodes in the network.
    
    .. attribute:: edges
    
        Number of edges in the network. 
    
    .. attribute:: repository
    
        The repository name (Social networks, Communication networks, ...).
    
    .. attribute:: description
    
        Detailed description of the network.
    
    """
    def __init__(self, name='', link='', type='', nodes='', edges='', repository='', description=''):
        self.name = name
        self.link = link
        self.type = type
        self.nodes = nodes
        self.edges = edges
        self.repository = repository
        self.description = description
        self._root =  Orange.utils.environ.buffer_dir + "/snap/"
        self._local_file = self._root + self.name + ".txt.gz"
        self._remote_file = "http://snap.stanford.edu/data/" + self.name + ".txt.gz"
        
    def read(self, progress_callback=None):
        """Read and return the network from file. Download the network to the 
        Orange home first if it was not jet downloaded. 
        
        :param progress_callback: a callback method to update a progress bar
        :type progress_callback: function(numblocks, blocksize, filesize)
        
        """
        
        if not self._is_downloaded():
            self._download(progress_callback)
        
        return readwrite.read(self._local_file)
    
    def _is_downloaded(self):
        if os.path.isfile(self._local_file):
            return True
        else:
            return False
        
    def _download(self, progress_callback=None):
        if not os.path.exists(self._root):
            os.makedirs(self._root)
            
        urllib.urlretrieve(self._remote_file, self._local_file, progress_callback)

class SNAPParser(HTMLParser):
    def __init__(self, *args, **kwargs):
        HTMLParser.__init__(self) 
        
        self.h3 = False
        self.tr = False
        self.td = False
        self.title = ''
        self.table = False
        self.networks = []
        
    def handle_starttag(self, tag, attrs):
        attrs = dict(attrs)
        
        if tag == 'h3':
            self.h3 = True
            
        if tag == 'table':
            self.table = True
            #self.networks = []
            
        if tag == 'tr':
            self.tr = True
            self.net_data = []
            
        if tag == 'td':
            self.td = True
            
        if tag == 'a' and self.td and self.tr:
            self.net_data.append(attrs.get('href', ''))
            
    def handle_data(self, data):
        if self.h3:
            self.title = data
            
        if self.tr and self.td:
            self.net_data.append(data)
            
    def handle_endtag(self, tag):
        if tag == 'h3':
            self.h3 = False
            
        if tag == 'table':
            self.table = False
            #self.repos[self.title] = self.networks
            
        if tag == 'tr':
            if len(self.net_data) == 6:
                self.networks.append(NetworkInfo(self.net_data[1], 
                     'http://snap.stanford.edu/data/' + self.net_data[0],
                     self.net_data[2],
                     self.net_data[3],
                     self.net_data[4],
                     self.title,
                     self.net_data[5]))
                
            self.tr = False
            
        if tag == 'td':
            self.td = False
    
class SNAP(object):
    """A collection of methods to access the information about networks in the 
    Stanford Large Network Dataset Collection.
    
    .. attribute:: network_list
    
        A list of networks on the `Stanford Large Network Dataset Collection web 
        site <http://snap.stanford.edu/data>`_. Each list item is an instance of the 
        :obj:`Orange.network.snap.NetworkInfo` class.
    
    """
    
    def __init__(self):
        self.network_list = []
        self.http = None
        
    def parse_snap(self, error, done_callback, progress_callback=None):
        if not error:
            #self.last_total = self.http.bytesAvailable()
            #if self.last_total <= 0:
            #    self.last_total = 24763
            
            src = str(self.http.readAll())
            snap_parser = SNAPParser()
            snap_parser.feed(src)
            
            self.network_list = snap_parser.networks
            done_callback(snap_parser.networks)
        else:
            done_callback(None)
        
        if progress_callback is not None:
            progress_callback(1,1)
        
    def get_network_list(self, done_callback=None, progress_callback=None):
        """Read the networks from the SNAP web site and populate the n
        etwork_list attribute. If done_callback is set, an asynchronous HTTP 
        request is made to the SNAP web site. If the done_callback is left None, 
        the HTTP request made is synchronous and the network_list is returned.
        
        :param done_callback: a callback method called when the network info is downloaded
        :type done_callback: function(bool)
        
        :param progress_callback: a callback method to update a progress bar
        :type progress_callback: function(done, total)
        
        """
        if done_callback == None:  
            conn = httplib.HTTPConnection("snap.stanford.edu")
            conn.request("GET", "/data/index.html")
            r1 = conn.getresponse()
            src = r1.read()
            snap_parser = SNAPParser()
            snap_parser.feed(src)
            self.network_list = snap_parser.networks
            return self.network_list
        else:
            import PyQt4.QtNetwork
            from PyQt4.QtCore import QObject, QString, SIGNAL

            self.http = PyQt4.QtNetwork.QHttp()
            
            QObject.connect(self.http, SIGNAL('done(bool)'), lambda error,\
                    done_callback=done_callback, progress_callback=progress_callback: \
                    self.parse_snap(error, done_callback, progress_callback))
            
            if progress_callback is not None:
                QObject.connect(self.http, SIGNAL('dataReadProgress(int,int)'), progress_callback)
            
            self.http.setHost('snap.stanford.edu')
            self.http.get('/data/index.html')
            
    def get_network(self, id):
        """Find and return the network by name. If no network is found, return 
        None. Call get_network_list before calling this method to populate the 
        network_list attribute.
        
        :param id: a name of the network in SNAP collection
        :type id: string
        
        """
        
        for network in self.network_list:
            if str(network.name) == str(id):
                return network
            
        return None



"""
*************************
Network Classes in Orange
*************************

Orange network classes provide methods for graph manipulation,
network analysis, and layout optimization. They are derived from
`NetworkX basic graph types <http://networkx.lanl.gov/reference/classes.html>`_
and :obj:`Orange.network.BaseGraph`.

We support four graph types: :obj:`Orange.network.Graph`,
:obj:`Orange.network.DiGraph`, :obj:`Orange.network.MultiGraph`, and
:obj:`Orange.network.MultiDiGraph`. Choose the graph type that matches the
structure of the graph you wish to represent.

Examples
========

Reading and Writing Networks
----------------------------

Network class reads and writes Pajek (.net), GML, and gpickle file formats.

:download:`network-read-nx.py <code/network-read-nx.py>`:

.. literalinclude:: code/network-read.py
    :lines: 5-6
    
Visualize Networks in Net Explorer Widget
-----------------------------------------

To display a network in Net Explorer widget write:

part of :download:`network-widget.py <code/network-widget.py>`

.. literalinclude:: code/network-widget.py
    :lines: 10-16
    
.. image:: files/network-explorer.png
    :width: 100%

"""

from pkg_resources import resource_filename
def networks():
    yield ('', resource_filename(__name__, 'networks'))

import networkx as nx

from network import *

import community
import snap



#
# OWHist.py
#
# the base for network histograms

import math
import numpy

import Orange
import OWGUI

from OWWidget import *
from OWGraph import *
from OWHist import *

class OWNxHist():

    def __init__(self, parent=None, type=0):
        self.parent = parent

        # set default settings
        self.spinLowerThreshold = 0
        self.spinLowerChecked = False
        self.spinUpperThreshold = 0
        self.spinUpperChecked = False
        self.netOption = 0
        self.dstWeight = 0
        self.kNN = 0
        self.andor = 0
        self.matrix = None
        self.excludeLimit = 2
        self.percentil = 0

        self.graph = None
        self.graph_matrix = None

    def addHistogramControls(self, parent=None):
        if parent is None:
            parent = self.controlArea

        boxGeneral = OWGUI.widgetBox(parent, box="Distance boundaries")

        ribg = OWGUI.widgetBox(boxGeneral, None, orientation="horizontal", addSpace=False)
        OWGUI.lineEdit(ribg, self, "spinLowerThreshold", "Lower", orientation='horizontal', callback=self.changeLowerSpin, valueType=float, enterPlaceholder=True, controlWidth=100)
        OWGUI.lineEdit(ribg, self, "spinUpperThreshold", "Upper    ", orientation='horizontal', callback=self.changeUpperSpin, valueType=float, enterPlaceholder=True, controlWidth=100)
        ribg.layout().addStretch(1)
        #ribg = OWGUI.radioButtonsInBox(boxGeneral, self, "andor", [], orientation='horizontal', callback = self.generateGraph)
        #OWGUI.appendRadioButton(ribg, self, "andor", "OR", callback = self.generateGraph)
        #b = OWGUI.appendRadioButton(ribg, self, "andor", "AND", callback = self.generateGraph)
        #b.setEnabled(False)
        #ribg.hide(False)

        ribg = OWGUI.widgetBox(boxGeneral, None, orientation="horizontal", addSpace=False)
        OWGUI.spin(ribg, self, "kNN", 0, 1000, 1, label="kNN   ", orientation='horizontal', callback=self.generateGraph, callbackOnReturn=1, controlWidth=100)
        OWGUI.doubleSpin(ribg, self, "percentil", 0, 100, 0.1, label="Percentile", orientation='horizontal', callback=self.setPercentil, callbackOnReturn=1, controlWidth=100)
        ribg.layout().addStretch(1)
        # Options
        self.attrColor = ""
        ribg = OWGUI.radioButtonsInBox(parent, self, "netOption", [], "Options", callback=self.generateGraph)
        OWGUI.appendRadioButton(ribg, self, "netOption", "All vertices", callback=self.generateGraph)
        hb = OWGUI.widgetBox(ribg, None, orientation="horizontal", addSpace=False)
        OWGUI.appendRadioButton(ribg, self, "netOption", "Large components only. Min nodes:", insertInto=hb, callback=self.generateGraph)
        OWGUI.spin(hb, self, "excludeLimit", 2, 100, 1, callback=(lambda h=True: self.generateGraph(h)))
        OWGUI.appendRadioButton(ribg, self, "netOption", "Largest connected component only", callback=self.generateGraph)
        OWGUI.appendRadioButton(ribg, self, "netOption", "Connected component with vertex")
        self.attribute = None
        self.attributeCombo = OWGUI.comboBox(parent, self, "attribute", box="Filter attribute", orientation='horizontal')#, callback=self.setVertexColor)

        ribg = OWGUI.radioButtonsInBox(parent, self, "dstWeight", [], "Distance -> Weight", callback=self.generateGraph)
        hb = OWGUI.widgetBox(ribg, None, orientation="horizontal", addSpace=False)
        OWGUI.appendRadioButton(ribg, self, "dstWeight", "Weight := distance", insertInto=hb, callback=self.generateGraph)
        OWGUI.appendRadioButton(ribg, self, "dstWeight", "Weight := 1 - distance", insertInto=hb, callback=self.generateGraph)

        self.label = ''
        self.searchString = OWGUI.lineEdit(self.attributeCombo.box, self, "label", callback=self.setSearchStringTimer, callbackOnType=True)
        self.searchStringTimer = QTimer(self)
        self.connect(self.searchStringTimer, SIGNAL("timeout()"), self.generateGraph)

        if str(self.netOption) != '3':
            self.attributeCombo.box.setEnabled(False)

    def setPercentil(self):
        if self.matrix is None or self.percentil <= 0:
            return

        self.spinLowerThreshold = self.histogram.minValue
        # flatten matrix, sort values and remove identities (self.matrix[i][i])
        vals = sorted(sum(self.matrix, ()))[self.matrix.dim:]
        ind = int(len(vals) * self.percentil / 100)
        self.spinUpperThreshold = vals[ind]
        self.generateGraph()

    def enableAttributeSelection(self):
        self.attributeCombo.box.setEnabled(True)

    def setSearchStringTimer(self):
        self.searchStringTimer.stop()
        self.searchStringTimer.start(750)

    def setMatrix(self, data):
        if data == None:
            self.matrix = None
            self.histogram.setValues([])
            self.attributeCombo.clear()
            self.generateGraph()
            return

        if not hasattr(data, "items") or data.items is None:
            setattr(data, "items", [i for i in range(data.dim)])

        self.matrix = data
        # draw histogram
        data.matrixType = orange.SymMatrix.Symmetric
        values = data.getValues()
        #print "values:",values
        self.histogram.setValues(values)

        low = min(values)
        upp = max(values)
        self.spinLowerThreshold = self.spinUpperThreshold = math.floor(low - (0.03 * (upp - low)))

        self.attributeCombo.clear()
        vars = []

        if hasattr(self.matrix, "items"):

            if isinstance(self.matrix.items, orange.ExampleTable):
                vars = list(self.matrix.items.domain.variables)

                metas = self.matrix.items.domain.getmetas(0)
                for i, var in metas.iteritems():
                    vars.append(var)

        self.icons = self.createAttributeIconDict()

        for var in vars:
            try:
                if var.varType != 7: # if not Orange.feature.Python
                    self.attributeCombo.addItem(self.icons[var.varType], unicode(var.name))
            except:
                print "Error adding", var, "to the attribute combo."

        self.setPercentil()
        self.generateGraph()

    def changeLowerSpin(self):
        self.percentil = 0

        if self.spinLowerThreshold < self.histogram.minValue:
            self.spinLowerThreshold = self.histogram.minValue
        elif self.spinLowerThreshold > self.histogram.maxValue:
            self.spinLowerThreshold = self.histogram.maxValue

        if self.spinLowerThreshold >= self.spinUpperThreshold:
            self.spinUpperThreshold = self.spinLowerThreshold

        self.generateGraph()

    def changeUpperSpin(self):
        self.percentil = 0

        if self.spinUpperThreshold < self.histogram.minValue:
            self.spinUpperThreshold = self.histogram.minValue
        elif self.spinUpperThreshold > self.histogram.maxValue:
            self.spinUpperThreshold = self.histogram.maxValue

        if self.spinUpperThreshold <= self.spinLowerThreshold:
            self.spinLowerThreshold = self.spinUpperThreshold

        self.generateGraph()

    def generateGraph(self, N_changed=False):
        self.searchStringTimer.stop()
        self.attributeCombo.box.setEnabled(False)
        self.error()
        matrix = None
        self.warning('')

        if N_changed:
            self.netOption = 1

        if self.matrix == None:
            if hasattr(self, "infoa"):
                self.infoa.setText("No data loaded.")
            if hasattr(self, "infob"):
                self.infob.setText("")
            if hasattr(self, "infoc"):
                self.infoc.setText("")
            self.pconnected = 0
            self.nedges = 0
            self.graph = None
            if hasattr(self, "sendSignals"):
                self.sendSignals()
            return

        #print len(self.histogram.yData), len(self.histogram.xData)
        nEdgesEstimate = 2 * sum([self.histogram.yData[i] for i, e in enumerate(self.histogram.xData) if self.spinLowerThreshold <= e <= self.spinUpperThreshold])

        if nEdgesEstimate > 200000:
            self.graph = None
            nedges = 0
            n = 0
            self.error('Estimated number of edges is too high (%d).' % nEdgesEstimate)
        else:
            graph = Orange.network.Graph()
            graph.add_nodes_from(range(self.matrix.dim))
            matrix = self.matrix

            if hasattr(self.matrix, "items") and self.matrix.items is not None:
                if type(self.matrix.items) == Orange.data.Table:
                    graph.set_items(self.matrix.items)
                else:
                    data = [[str(x)] for x in self.matrix.items]
                    items = Orange.data.Table(Orange.data.Domain(Orange.feature.String('label'), 0), data)
                    graph.set_items(items)

            # set the threshold
            # set edges where distance is lower than threshold
            self.warning(0)
            if self.kNN >= self.matrix.dim:
                self.warning(0, "kNN larger then supplied distance matrix dimension. Using k = %i" % (self.matrix.dim - 1))
            #nedges = graph.fromDistanceMatrix(self.matrix, self.spinLowerThreshold, self.spinUpperThreshold, min(self.kNN, self.matrix.dim - 1), self.andor)
            edge_list = Orange.network.GraphLayout().edges_from_distance_matrix(self.matrix, self.spinLowerThreshold, self.spinUpperThreshold, min(self.kNN, self.matrix.dim - 1))
            if self.dstWeight == 1:
                graph.add_edges_from(((u, v, {'weight':1 - d}) for u, v, d in edge_list))
            else:
                graph.add_edges_from(((u, v, {'weight':d}) for u, v, d in edge_list))

            # exclude unconnected
            if str(self.netOption) == '1':
                components = [x for x in Orange.network.nx.algorithms.components.connected_components(graph) if len(x) >= self.excludeLimit]
                if len(components) > 0:
                    include = reduce(lambda x, y: x + y, components)
                    if len(include) > 1:
                        self.graph = graph.subgraph(include)
                        matrix = self.matrix.getitems(include)
                    else:
                        self.graph = None
                        matrix = None
                else:
                    self.graph = None
                    matrix = None
            # largest connected component only
            elif str(self.netOption) == '2':
                component = Orange.network.nx.algorithms.components.connected_components(graph)[0]
                if len(component) > 1:
                    self.graph = graph.subgraph(component)
                    matrix = self.matrix.getitems(component)
                else:
                    self.graph = None
                    matrix = None
            # connected component with vertex by label
            elif str(self.netOption) == '3':
                self.attributeCombo.box.setEnabled(True)
                self.graph = None
                matrix = None
                #print self.attributeCombo.currentText()
                if self.attributeCombo.currentText() != '' and self.label != '':
                    components = Orange.network.nx.algorithms.components.connected_components(graph)

                    txt = self.label.lower()
                    #print 'txt:',txt
                    nodes = [i for i, values in enumerate(self.matrix.items) if txt in str(values[str(self.attributeCombo.currentText())]).lower()]
                    #print "nodes:",nodes
                    if len(nodes) > 0:
                        vertices = []
                        for component in components:
                            for node in nodes:
                                if node in component:
                                    if len(component) > 0:
                                        vertices.extend(component)

                        if len(vertices) > 0:
                            #print "n vertices:", len(vertices), "n set vertices:", len(set(vertices))
                            vertices = list(set(vertices))
                            self.graph = graph.subgraph(include)
                            matrix = self.matrix.getitems(vertices)
            else:
                self.graph = graph

        if matrix != None:
            matrix.setattr("items", self.graph.items())
        self.graph_matrix = matrix

        if self.graph is None:
            self.pconnected = 0
            self.nedges = 0
        else:
            self.pconnected = self.graph.number_of_nodes()
            self.nedges = self.graph.number_of_edges()
        if hasattr(self, "infoa"):
            self.infoa.setText("Matrix size: %d" % self.matrix.dim)
        if hasattr(self, "infob"):
            self.infob.setText("Graph nodes: %d (%3.1f%%)" % (self.pconnected,
                self.pconnected / float(self.matrix.dim) * 100))
        if hasattr(self, "infoc"):
            self.infoc.setText("Graph edges: %d (%.2f edges/node)" % (
                self.nedges, self.nedges / float(self.pconnected)
                if self.pconnected else 0))

        #print 'self.graph:',self.graph+
        if hasattr(self, "sendSignals"):
            self.sendSignals()

        self.histogram.setBoundary(self.spinLowerThreshold, self.spinUpperThreshold)




#!/usr/bin/env python

try:
    import distribute_setup
    distribute_setup.use_setuptools()
except ImportError:
    # For documentation we load setup.py to get version
    # so it does not matter if importing fails
    pass

import os

from setuptools import setup, find_packages

NAME = 'Orange-Network'
DOCUMENTATION_NAME = 'Orange Network'

VERSION = '0.2.11'

DESCRIPTION = 'Orange Network add-on for Orange data mining software package.'
LONG_DESCRIPTION  = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
AUTHOR = 'Miha Stajdohar, FRI UL'
AUTHOR_EMAIL = 'miha.stajdohar@gmail.com'
URL = 'http://orange.biolab.si/addons/'
DOWNLOAD_URL = 'https://bitbucket.org/mstajdohar/orange-network/downloads'
LICENSE = 'GPLv3'

KEYWORDS = (
    'network',
    'network analysis',
    'network layout',
    'network visualization',
    'data mining',
    'machine learning',
    'artificial intelligence',
    'orange',
    'orange add-on',
)

CLASSIFIERS = (
    'Development Status :: 4 - Beta',
    'Environment :: X11 Applications :: Qt',
    'Environment :: Console',
    'Environment :: Plugins',
    'Programming Language :: Python',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Operating System :: OS Independent',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'Topic :: Scientific/Engineering :: Visualization',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Intended Audience :: Education',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',
)

PACKAGES = find_packages(
    exclude = ('*.tests', '*.tests.*', 'tests.*', 'tests'),
)
 
PACKAGE_DATA = {
}

SETUP_REQUIRES = (
    'setuptools',
)

INSTALL_REQUIRES = (
    'networkx',
    'Orange',
    'setuptools',
#    'numpy',
#    'scipy',      # Requires Fortran compiler
),

EXTRAS_REQUIRE = {
    # Dependencies which are problematic to install automatically
    'GUI': (
        'PyQt4',
    ),
    'reST': (
        'numpydoc',
    ),
}

DEPENDENCY_LINKS = (
)

ENTRY_POINTS = {
    'orange.addons': (
        'network = _network',
    ),
    'orange.widgets': (
        'Network = _network.widgets',
    ),
    'orange.data.io.search_paths': (
        'network = _network:networks',
    ),
}

if __name__ == '__main__':
    setup(
        name = NAME,
        version = VERSION,
        description = DESCRIPTION,
        long_description = LONG_DESCRIPTION,
        author = AUTHOR,
        author_email = AUTHOR_EMAIL,
        url = URL,
        download_url = DOWNLOAD_URL,
        license = LICENSE,
        keywords = KEYWORDS,
        classifiers = CLASSIFIERS,
        packages = PACKAGES,
        package_data = PACKAGE_DATA,
        setup_requires = SETUP_REQUIRES,
        install_requires = INSTALL_REQUIRES,
        extras_require = EXTRAS_REQUIRE,
        dependency_links = DEPENDENCY_LINKS,
        entry_points = ENTRY_POINTS,
        include_package_data = True,
        zip_safe = False,
    )



CIRCLE = 0
SQUARE = 1
ROUND_RECT = 2

NOTHING = 0
ZOOMING = 1
SELECT_RECTANGLE = 2
SELECT_POLYGON = 3
MOVE_SELECTION = 100

import Orange

from numpy import *
from OWGraph import *
from orngScaleScatterPlotData import *

class NodeItem():
    def __init__(self, index=-1):
        self.index = index
        self.marked = False
        self.show = True
        self.highlight = False
        self.selected = False
        self.label = []
        self.tooltip = []
        self.uuid = None
        
        self.image = None
        self.pen = QPen(Qt.blue, 1)
        self.pen.setJoinStyle(Qt.RoundJoin)
        self.nocolor = Qt.white
        self.color = Qt.blue
        self.size = 5
        self.style = 1
    
class EdgeItem():
    def __init__(self, u=None, v=None, weight=0, arrowu=0, arrowv=0, 
                 links_index=None, label=[]):
        self.u = u
        self.v = v
        self.links_index = links_index
        self.arrowu = arrowu
        self.arrowv = arrowv
        self.weight = weight
        self.label = label

        self.pen = QPen(Qt.lightGray, 1)
        self.pen.setCapStyle(Qt.RoundCap)

class NetworkCurve(QwtPlotCurve):
  def __init__(self, parent, pen=QPen(Qt.black), xData=None, yData=None):
      QwtPlotCurve.__init__(self, "Network Curve")

      self.coors = {}
      self.vertices = {}
      self.edges = []
      self.setItemAttribute(QwtPlotItem.Legend, 0)
      self.showEdgeLabels = 0

  def move_selected_nodes(self, dx, dy):
    selected = self.get_selected_nodes()
    
    self.coors[0][selected] = self.coors[0][selected] + dx
    self.coors[1][selected] = self.coors[1][selected] + dy
      
    self.setData(self.coors[0], self.coors[1])
    return selected
  
  def set_node_color(self, v, color):
      pen = self.vertices[v].pen
      self.vertices[v].color = color
      self.vertices[v].pen = QPen(color, pen.width())

  def set_nodes_color(self, vertices, color):
      for v in vertices:
          v.color = color
          v.pen = QPen(color, v.pen.width())
      
  def set_edge_color(self, index, color, nocolor=0):
      pen = self.edges[index].pen
      if nocolor:
        color.setAlpha(0)
      self.edges[index].pen = QPen(color, pen.width())
      self.edges[index].pen.setCapStyle(Qt.RoundCap)
  
  def get_selected_nodes(self):
    return [vertex.index for vertex in self.vertices.itervalues() if vertex.selected]

  def get_unselected_nodes(self):
    return [vertex.index for vertex in self.vertices.itervalues() if not vertex.selected]

  def get_marked_nodes(self):
    return [vertex.index for vertex in self.vertices.itervalues() if vertex.marked]
  
  def set_marked_nodes(self, vertices):
    for vertex in self.vertices.itervalues():
      if vertex.index in vertices:
        vertex.marked = True
      else:
        vertex.marked = False
        
  def mark_to_sel(self):
    for vertex in self.vertices.itervalues():
      if vertex.marked == True:
          vertex.selected = True
          
  def sel_to_mark(self):
    for vertex in self.vertices.itervalues():
      if vertex.selected == True:
          vertex.selected = False
          vertex.marked = True
  
  def unmark(self):
    for vertex in self.vertices.itervalues():
      vertex.marked = False
      
  def unselect(self):
    for vertex in self.vertices.itervalues():
        vertex.selected = False
        
  def set_hidden_nodes(self, nodes):
    for vertex in self.vertices.itervalues():
      if vertex.index in nodes:
        vertex.show = False
      else:
        vertex.show = True
      
  def hide_selected_nodes(self):
    for vertex in self.vertices.itervalues():
      if vertex.selected:
        vertex.show = False
  
  def hide_unselected_nodes(self):
    for vertex in self.vertices.itervalues():
      if not vertex.selected:
        vertex.show = False
    
  def show_all_vertices(self):
    for vertex in self.vertices.itervalues():
      vertex.show = True
    
  def changed(self):
      self.itemChanged()

  def draw(self, painter, xMap, yMap, rect):
    for edge in self.edges:
      if edge.u.show and edge.v.show:
        painter.setPen(edge.pen)

        px1 = xMap.transform(self.coors[edge.u.index][0])   #ali pa tudi self.x1, itd
        py1 = yMap.transform(self.coors[edge.u.index][1])
        px2 = xMap.transform(self.coors[edge.v.index][0])
        py2 = yMap.transform(self.coors[edge.v.index][1])
        
        painter.drawLine(px1, py1, px2, py2)
        
        d = 12
        #painter.setPen(QPen(Qt.lightGray, 1))
        painter.setBrush(Qt.lightGray)
        if edge.arrowu:
            x = px2 - px1
            y = py2 - py1
            
            fi = math.atan2(y, x) * 180 * 16 / math.pi 

            if not fi is None:
                # (180*16) - fi - (20*16), (40*16)
                rect = QRectF(px2 - d, py2 - d, 2 * d, 2 * d)
                painter.drawPie(rect, 2560 - fi, 640)
                
        if edge.arrowv:
            x = px2 - px1
            y = py2 - py1
            
            fi = math.atan2(y, x) * 180 * 16 / math.pi 
            if not fi is None:
                # (180*16) - fi - (20*16), (40*16)
                rect = QRectF(px2 - d, py2 - d, 2 * d, 2 * d)
                painter.drawPie(rect, 2560 - fi, 640)
                
        if self.showEdgeLabels and len(edge.label) > 0:
            lbl = ', '.join(edge.label)
            x = (px1 + px2) / 2
            y = (py1 + py2) / 2
            
            th = painter.fontMetrics().height()
            tw = painter.fontMetrics().width(lbl)
            r = QRect(x - tw / 2, y - th / 2, tw, th)
            painter.fillRect(r, QBrush(Qt.white))
            painter.drawText(r, Qt.AlignHCenter + Qt.AlignVCenter, lbl)
    
    for key, vertex in self.vertices.iteritems():
      if vertex.show:
        pX = xMap.transform(self.coors[vertex.index][0])   #dobimo koordinati v pikslih (tipa integer)
        pY = yMap.transform(self.coors[vertex.index][1])   #ki se stejeta od zgornjega levega kota canvasa
        if vertex.selected:    
          painter.setPen(QPen(Qt.yellow, 3))
          painter.setBrush(vertex.color)
          rect = QRectF(pX - (vertex.size + 4) / 2, pY - (vertex.size + 4) / 2, vertex.size + 4, vertex.size + 4)
          painter.drawEllipse(rect)
        elif vertex.marked:
          painter.setPen(vertex.pen)
          painter.setBrush(vertex.color)
          rect = QRectF(pX - vertex.size / 2, pY - vertex.size / 2, vertex.size, vertex.size)
          painter.drawEllipse(rect)
        else:
          painter.setPen(vertex.pen)
          painter.setBrush(vertex.nocolor)
          rect = QRectF(pX - vertex.size / 2, pY - vertex.size / 2, vertex.size, vertex.size)
          painter.drawEllipse(rect)
        
  def closest_node(self, px, py):
    ndx = min(self.coors, key=lambda x: abs(self.coors[x][0]-px) + abs(self.coors[x][1]-py))
    return ndx, math.sqrt((self.coors[ndx][0]-px)**2 + (self.coors[ndx][0]-px)**2)

  def get_nodes_in_rect(self, x1, y1, x2, y2):
      if x1 > x2:
          x1, x2 = x2, x1
      if y1 > y2:
          y1, y2 = y2, y1
      return [key for key in self.coors if x1 < self.coors[key][0] < x2 and y1 < self.coors[key][1] < y2]
        
class OWNxCanvas(OWGraph):
    def __init__(self, master, parent=None, name="None"):
        OWGraph.__init__(self, parent, name)
        self.master = master
        self.parent = parent
        self.labelText = []
        self.tooltipText = []
        #self.vertices_old = {}         # distionary of nodes (orngIndex: vertex_objekt)
        #self.edges_old = {}            # distionary of edges (curveKey: edge_objekt)
        #self.vertices = []
        #self.edges = []
        self.indexPairs = {}       # distionary of type CurveKey: orngIndex   (for nodes)
        #self.selection = []        # list of selected nodes (indices)
        self.markerKeys = {}       # dictionary of type NodeNdx : markerCurveKey
        self.tooltipKeys = {}      # dictionary of type NodeNdx : tooltipCurveKey
        self.graph = None
        self.layout = None
        self.vertexDegree = []     # seznam vozlisc oblike (vozlisce, stevilo povezav), sortiran po stevilu povezav
        self.edgesKey = -1
        #self.vertexSize = 6
        self.enableXaxis(0)
        self.enableYLaxis(0)
        self.state = NOTHING  #default je rocno premikanje
        self.hiddenNodes = []
        self.markedNodes = set()
        self.markWithRed = False
        self.circles = []
        self.tooltipNeighbours = 2
        self.selectionNeighbours = 2
        self.freezeNeighbours = False
        self.labelsOnMarkedOnly = 0
        self.enableWheelZoom = 1
        self.optimizing = 0
        self.stopOptimizing = 0
        self.insideview = 0
        self.insideviewNeighbours = 2
        self.enableGridXB(False)
        self.enableGridYL(False)
        self.renderAntialiased = 1
        self.sendMarkedNodes = None
        self.showEdgeLabels = 0
        self.showDistances = 0
        self.showMissingValues = 0
        
        self.showWeights = 0
        self.showIndexes = 0
        self.minEdgeWeight = sys.maxint
        self.maxEdgeWeight = 0
        self.maxEdgeSize = 1
        
        self.maxVertexSize = 5
        self.minVertexSize = 5
        self.invertEdgeSize = 0
        self.showComponentAttribute = None
        self.forceVectors = None
        self.appendToSelection = 1
        self.fontSize = 12
             
        self.setAxisAutoScale(self.xBottom)
        self.setAxisAutoScale(self.yLeft)
        
        self.networkCurve = NetworkCurve(self)
        self.callbackMoveVertex = None
        self.callbackSelectVertex = None
        self.minComponentEdgeWidth = 0
        self.maxComponentEdgeWidth = 0
        self.items_matrix = None
        self.controlPressed = False
        self.altPressed = False
        self.items = None
        self.links = None
        
        self.setFocusPolicy(Qt.StrongFocus)
        
    def getSelection(self):
      return self.networkCurve.get_selected_nodes()
    
    def get_marked_nodes(self):
      return self.networkCurve.get_marked_nodes()
        
    def getVertexSize(self, index):
        return 6
        
    def set_hidden_nodes(self, nodes):
        self.networkCurve.set_hidden_nodes(nodes)
    
    def hide_selected_nodes(self):
      self.networkCurve.hide_selected_nodes()
      self.drawPlotItems()
      
    def hide_unselected_nodes(self):
      self.networkCurve.hide_unselected_nodes()
      self.drawPlotItems()
      
    def show_all_vertices(self):
      self.networkCurve.show_all_vertices()
      self.drawPlotItems()
      
    def optimize(self, frSteps):
        qApp.processEvents()
        tolerance = 5
        initTemp = 100
        breakpoints = 20
        k = int(frSteps / breakpoints)
        o = frSteps % breakpoints
        iteration = 0
        coolFactor = exp(log(10.0 / 10000.0) / frSteps)
        #print coolFactor
        if k > 0:
            while iteration < breakpoints:
                initTemp = self.layout.fruchtermanReingold(k, initTemp, coolFactor, self.hiddenNodes)
                iteration += 1
                qApp.processEvents()
                self.updateCanvas()
    
            initTemp = self.layout.fruchtermanReingold(o, initTemp, coolFactor, self.hiddenNodes)
            qApp.processEvents()
            self.updateCanvas()
        else:
            while iteration < o:
                initTemp = self.layout.fruchtermanReingold(1, initTemp, coolFactor, self.hiddenNodes)
                iteration += 1
                qApp.processEvents()
                self.updateCanvas()
                
    def markedToSelection(self):
        self.networkCurve.mark_to_sel()
        self.drawPlotItems()
        
    def selectionToMarked(self):
        self.networkCurve.sel_to_mark()
        self.drawPlotItems()
        
        if self.sendMarkedNodes != None:
            self.sendMarkedNodes(self.networkCurve.get_marked_nodes())
        
    def removeSelection(self, replot=True):
        self.networkCurve.unselect()
        
        if replot:
          self.replot()
    
    def selectNeighbours(self, sel, nodes, depth, maxdepth):
        #print "list: " + str(sel)
        #print "nodes: " + str(nodes)
        sel.update(nodes)
        if depth < maxdepth:
            for i in nodes:
                neighbours = set(self.graph.neighbors(i))
                #print "neighbours: " + str(neighbours)
                self.selectNeighbours(sel, neighbours - sel, depth + 1, maxdepth)
        
    def getSelectedExamples(self):
        return self.networkCurve.get_selected_nodes()
        
    def getUnselectedExamples(self):
        return self.networkCurve.get_unselected_nodes()
    
    def getSelectedGraph(self):
      selection = self.networkCurve.get_selected_nodes()
      
      if len(selection) == 0:
          return None
      
      subgraph = self.graph.subgraph(selection)
      subnet = Network(subgraph)
      return subnet
    
    def get_selected_nodes(self):
      return self.networkCurve.get_selected_nodes()
    
    def getNeighboursUpTo(self, ndx, dist):
        newNeighbours = neighbours = set([ndx])
        for d in range(dist):
            tNewNeighbours = set()
            for v in newNeighbours:
                tNewNeighbours |= set(self.graph.neighbors(v))
            newNeighbours = tNewNeighbours - neighbours
            neighbours |= newNeighbours
        return neighbours
     
    def markSelectionNeighbours(self):
        if not self.freezeNeighbours and self.selectionNeighbours:
            toMark = set()
            for ndx in self.networkCurve.get_selected_nodes():
                toMark |= self.getNeighboursUpTo(ndx, self.selectionNeighbours)
            
            self.networkCurve.set_marked_nodes(toMark)
            self.drawPlotItems()
                
        elif not self.freezeNeighbours and self.selectionNeighbours == 0:
            self.networkCurve.set_marked_nodes(self.networkCurve.get_selected_nodes())
            self.drawPlotItems()
            
        if self.sendMarkedNodes != None:
            self.sendMarkedNodes(self.networkCurve.get_marked_nodes())
                
    def unmark(self):
      self.networkCurve.unmark()
      self.drawPlotItems(replot=0)
      
      if self.sendMarkedNodes != None:
            self.sendMarkedNodes([])
            
    def set_marked_nodes(self, vertices):
      self.networkCurve.set_marked_nodes(vertices)
      self.drawPlotItems(replot=0)
      
      if self.sendMarkedNodes != None:
            self.sendMarkedNodes(self.networkCurve.get_marked_nodes())
        
    def activateMoveSelection(self):
        self.state = MOVE_SELECTION
    
    def mouseMoveEvent(self, event):
        if not self.graph:
          return
          
        if self.mouseCurrentlyPressed and self.state == MOVE_SELECTION and self.GMmouseMoveEvent != None:
            newX = self.invTransform(2, event.pos().x())
            newY = self.invTransform(0, event.pos().y())
            
            dx = newX - self.invTransform(2, self.GMmouseMoveEvent.x())
            dy = newY - self.invTransform(0, self.GMmouseMoveEvent.y())
            movedVertices = self.networkCurve.move_selected_nodes(dx, dy)
            
            self.GMmouseMoveEvent.setX(event.pos().x())  #zacetni dogodek postane trenutni
            self.GMmouseMoveEvent.setY(event.pos().y())
            
            self.drawPlotItems(replot=1, vertices=movedVertices)
            if self.callbackMoveVertex:
                self.callbackMoveVertex()
        else:
            OWGraph.mouseMoveEvent(self, event)
                
        if not self.freezeNeighbours and self.tooltipNeighbours:
            px = self.invTransform(2, event.x())
            py = self.invTransform(0, event.y())   
            ndx, mind = self.networkCurve.closest_node(px, py)
            dX = self.transform(QwtPlot.xBottom, self.networkCurve.coors[ndx][0]) - event.x()
            dY = self.transform(QwtPlot.yLeft,   self.networkCurve.coors[ndx][1]) - event.y()
            # transform to pixel distance
            distance = math.sqrt(dX**2 + dY**2) 
              
            if ndx != -1 and distance <= self.networkCurve.vertices[ndx].size:
                toMark = set(self.getNeighboursUpTo(ndx, self.tooltipNeighbours))
                self.networkCurve.set_marked_nodes(toMark)
                self.drawPlotItems()
                
                if self.sendMarkedNodes != None:
                    self.sendMarkedNodes(self.networkCurve.get_marked_nodes())
            else:
                self.networkCurve.unmark()
                self.drawPlotItems()
                
                if self.sendMarkedNodes != None:
                    self.sendMarkedNodes([])
        
        if self.showDistances:
            selection = self.networkCurve.get_selected_nodes()
            if len(selection) > 0:
                px = self.invTransform(2, event.x())
                py = self.invTransform(0, event.y())  
                 
                v, mind = self.networkCurve.closest_node(px, py)
                dX = self.transform(QwtPlot.xBottom, self.networkCurve.coors[v][0]) - event.x()
                dY = self.transform(QwtPlot.yLeft,   self.networkCurve.coors[v][1]) - event.y()
                # transform to pixel distance
                distance = math.sqrt(dX**2 + dY**2)               
                if v != -1 and distance <= self.networkCurve.vertices[v].size:
                    if self.items_matrix == None:
                        dst = 'vertex distance signal not set'
                    else:
                        dst = 0
                        for u in selection:
                            dst += self.items_matrix[u, v]
                        dst = dst / len(selection)
                        
                    self.showTip(event.pos().x(), event.pos().y(), str(dst))
                    self.replot()
    
    def mousePressEvent(self, event):
      if self.graph is None:
          return
          
      #self.grabKeyboard()
      self.mouseSelectedVertex = 0
      self.GMmouseMoveEvent = None
      
      if self.state == MOVE_SELECTION:
        self.mouseCurrentlyPressed = 1
        #if self.isPointSelected(self.invTransform(self.xBottom, event.pos().x()), self.invTransform(self.yLeft, event.pos().y())) and self.selection != []:
        #  self.GMmouseStartEvent = QPoint(event.pos().x(), event.pos().y())
        #else:
          # button pressed outside selected area or there is no area
        self.selectVertex(event.pos())
        self.GMmouseStartEvent = QPoint(event.pos().x(), event.pos().y())
        self.replot()
      elif self.state == SELECT_RECTANGLE:
          self.GMmouseStartEvent = QPoint(event.pos().x(), event.pos().y())
          
          if self.clickedSelectedOnVertex(event.pos()):
              self.mouseSelectedVertex = 1
              self.mouseCurrentlyPressed = 1
              self.state = MOVE_SELECTION
              self.GMmouseMoveEvent = QPoint(event.pos().x(), event.pos().y())
          elif self.clickedOnVertex(event.pos()):
              self.mouseSelectedVertex = 1
              self.mouseCurrentlyPressed = 1
          else:
              OWGraph.mousePressEvent(self, event)  
      else:
          OWGraph.mousePressEvent(self, event)     
    
    def mouseReleaseEvent(self, event):  
        if self.graph is None:
          return
          
        #self.releaseKeyboard()
        if self.state == MOVE_SELECTION:
            self.state = SELECT_RECTANGLE
            self.mouseCurrentlyPressed = 0
            
            self.moveGroup = False
            #self.GMmouseStartEvent=None
            
        if self.state == SELECT_RECTANGLE:
            x1 = self.invTransform(2, self.GMmouseStartEvent.x())
            y1 = self.invTransform(0, self.GMmouseStartEvent.y())
            
            x2 = self.invTransform(2, event.pos().x())
            y2 = self.invTransform(0, event.pos().y())
            
            
            if self.mouseSelectedVertex == 1 and x1 == x2 and y1 == y2 and self.selectVertex(self.GMmouseStartEvent):
                QwtPlot.mouseReleaseEvent(self, event)
            elif self.mouseSelectedVertex == 0:
                 
                selection = self.networkCurve.get_nodes_in_rect(x1, y1, x2, y2)
    
                def selectVertex(ndx):
                    if self.networkCurve.vertices[ndx].show:
                        self.networkCurve.vertices[ndx].selected = True
                        
                map(selectVertex, selection)
                
                if len(selection) == 0 and x1 == x2 and y1 == y2:
                    self.removeSelection()
                    self.unmark()
            
                self.markSelectionNeighbours()
                OWGraph.mouseReleaseEvent(self, event)
                self.removeAllSelections()
    
        elif self.state == SELECT_POLYGON:
                OWGraph.mouseReleaseEvent(self, event)
                if self.tempSelectionCurve == None:   #if OWVisGraph closed polygon
                    self.selectVertices()
        else:
            OWGraph.mouseReleaseEvent(self, event)
            
        self.mouseCurrentlyPressed = 0
        self.moveGroup = False
            
        if self.callbackSelectVertex != None:
            self.callbackSelectVertex()
    
    def keyPressEvent(self, e):
        if self.graph is None:
          return
          
        if e.key() == 87 or e.key() == 81:
            selection = [v.index for v in self.networkCurve.vertices.itervalues() if v.selected]
            if len(selection) > 0:
                phi = [math.pi / -180 if e.key() == 87 else math.pi / 180]
                self.layout.rotate_vertices([selection], phi)
                self.drawPlotItems(replot=1)
            
        if e.key() == Qt.Key_Control:
            self.controlPressed = True
        
        elif e.key() == Qt.Key_Alt:
            self.altPressed = True
            
        if e.text() == "f":
            self.graph.freezeNeighbours = not self.graph.freezeNeighbours
        
        OWGraph.keyPressEvent(self, e)
            
    def keyReleaseEvent(self, e):
        if e.key() == Qt.Key_Control:
            self.controlPressed = False
        
        elif e.key() == Qt.Key_Alt:
            self.altPressed = False
        
        OWGraph.keyReleaseEvent(self, e)
        
    def clickedSelectedOnVertex(self, pos):
        min = 1000000
        ndx = -1
    
        px = self.invTransform(2, pos.x())
        py = self.invTransform(0, pos.y())   
    
        ndx, min = self.networkCurve.closest_node(px, py)
        dX = self.transform(QwtPlot.xBottom, self.networkCurve.coors[ndx][0]) - pos.x()
        dY = self.transform(QwtPlot.yLeft,   self.networkCurve.coors[ndx][1]) - pos.y()
        # transform to pixel distance
        distance = math.sqrt(dX**2 + dY**2)
        
        #self.networkCurve
        
        if ndx != -1 and distance <= self.networkCurve.vertices[ndx].size:
            return self.networkCurve.vertices[ndx].selected
        else:
            return False
        
    def clickedOnVertex(self, pos):
        min = 1000000
        ndx = -1
    
        px = self.invTransform(2, pos.x())
        py = self.invTransform(0, pos.y())   
    
        ndx, min = self.networkCurve.closest_node(px, py)
        dX = self.transform(QwtPlot.xBottom, self.networkCurve.coors[ndx][0]) - pos.x()
        dY = self.transform(QwtPlot.yLeft,   self.networkCurve.coors[ndx][1]) - pos.y()
        # transform to pixel distance
        distance = math.sqrt(dX**2 + dY**2)
        if ndx != -1 and distance <= self.networkCurve.vertices[ndx].size:
            return True
        else:
            return False
                
    def selectVertex(self, pos):
        min = 1000000
        ndx = -1
    
        px = self.invTransform(2, pos.x())
        py = self.invTransform(0, pos.y())   
    
        ndx, min = self.networkCurve.closest_node(px, py)
        
        dX = self.transform(QwtPlot.xBottom, self.networkCurve.coors[ndx][0]) - pos.x()
        dY = self.transform(QwtPlot.yLeft,   self.networkCurve.coors[ndx][1]) - pos.y()
        # transform to pixel distance
        distance = math.sqrt(dX**2 + dY**2)
        if ndx != -1 and distance <= self.networkCurve.vertices[ndx].size:
            if not self.appendToSelection and not self.controlPressed:
                self.removeSelection()
                      
            if self.insideview:
                self.networkCurve.unselect()
                self.networkCurve.vertices[ndx].selected = not self.networkCurve.vertices[ndx].selected
                self.optimize(100)
                
                self.markSelectionNeighbours()
            else:
                self.networkCurve.vertices[ndx].selected = not self.networkCurve.vertices[ndx].selected
                self.markSelectionNeighbours()
            
            return True  
        else:
            return False
            self.removeSelection()
            self.unmark()
    
    def updateData(self):
        if self.graph is None:
            return
        
        self.removeDrawingCurves(removeLegendItems=0)
        self.tips.removeAll()
        
        if self.items_matrix and self.minComponentEdgeWidth > 0 and self.maxComponentEdgeWidth > 0:          
            components = Orange.network.nx.algorithms.components.connected_components(self.graph)
            matrix = self.items_matrix.avgLinkage(components)
            
            edges = set()
            for u in range(matrix.dim):
                neighbours = matrix.getKNN(u, 2)
                for v in neighbours:
                    if v < u:
                        edges.add((v, u))
                    else:
                        edges.add((u, v))
            edges = list(edges)
    # show 2n strongest edges
    #          vals = matrix.getValues()
    #          vals = zip(vals, range(len(vals)))
    #          count = 0
    #          add = 0
    #          for i in range(matrix.dim):
    #              add += i + 1
    #              for j in range(i+1, matrix.dim):
    #                  v, ind = vals[count]
    #                  ind += add
    #                  vals[count] = (v, ind)
    #                  count += 1
    #          vals.sort(reverse=0)
    #          vals = vals[:2 * matrix.dim]
    #          edges = [(ind / matrix.dim, ind % matrix.dim) for v, ind in vals]
    #          print "number of component edges:", len(edges), "number of components:", len(components)
#            components_c = [(sum(self.networkCurve.coors[c][0]) / len(c), sum(self.networkCurve.coors[c][1]) / len(c)) for c in components]
#            weights = [1 - matrix[u,v] for u,v in edges]
#            
#            max_weight = max(weights)
#            min_weight = min(weights)
#            span_weights = max_weight - min_weight
#          
#            for u,v in edges:
#                x = [components_c[u][0], components_c[v][0]]
#                y = [components_c[u][1], components_c[v][1]]
#                w = ((1 - matrix[u,v]) - min_weight) / span_weights * (self.maxComponentEdgeWidth - self.minComponentEdgeWidth) + self.minComponentEdgeWidth
#                
#                pen = QPen()
#                pen.setWidth(int(w))
#                pen.setBrush(QColor(50,200,255,15))
#                pen.setCapStyle(Qt.FlatCap)
#                pen.setJoinStyle(Qt.RoundJoin)
#                self.addCurve("component_edges", Qt.green, Qt.green, 0, style=QwtPlotCurve.Lines, symbol = QwtSymbol.NoSymbol, xData=x, yData=y, pen=pen, showFilledSymbols=False)
        
        
        self.networkCurve.setData(self.layout.coors[0], self.layout.coors[1])
        
        if self.insideview == 1:
            selection = self.networkCurve.get_selected_nodes()
            if len(selection) >= 1:
                visible = set()
                visible |= set(selection)
                visible |= self.getNeighboursUpTo(selection[0], self.insideviewNeighbours)
                self.networkCurve.set_hidden_nodes(set(range(self.graph.number_of_nodes())) - visible)
    
        edgesCount = 0
        
        if self.forceVectors != None:
            for v in self.forceVectors:
                self.addCurve("force", Qt.white, Qt.green, 1, style=QwtPlotCurve.Lines, xData=v[0], yData=v[1], showFilledSymbols=False)
        
        for r in self.circles:
            step = 2 * pi / 64;
            fi = 0
            x = []
            y = []
            for i in range(65):
                x.append(r * cos(fi) + 5000)
                y.append(r * sin(fi) + 5000)
                fi += step
                
            self.addCurve("radius", Qt.white, Qt.green, 1, style=QwtPlotCurve.Lines, xData=x, yData=y, showFilledSymbols=False)
        
        if self.renderAntialiased:
            self.networkCurve.setRenderHint(QwtPlotItem.RenderAntialiased)
        else:
            self.networkCurve.setRenderHint(QwtPlotItem.RenderAntialiased, False)
      
        self.networkCurve.showEdgeLabels = self.showEdgeLabels
        self.networkCurve.attach(self)
        self.drawPlotItems(replot=0)
        
        #self.zoomExtent()
        
    def drawPlotItems(self, replot=1, vertices=[]):
        if len(vertices) > 0:
            for vertex in vertices:
                x1 = float(self.networkCurve.coors[vertex][0])
                y1 = float(self.networkCurve.coors[vertex][1])
                
                if vertex in self.markerKeys:
                    mkey = self.markerKeys[vertex]
                    mkey.setValue(x1, y1)
              
                if 'index ' + str(vertex) in self.markerKeys:
                    mkey = self.markerKeys['index ' + str(vertex)]
                    mkey.setValue(x1, y1)
                
                if vertex in self.tooltipKeys:
                    tkey = self.tooltipKeys[vertex]
                    self.tips.positions[tkey] = (x1, y1, 0, 0)
        else:
            self.markerKeys = {}
            self.removeMarkers()
            self.drawLabels()
            self.drawToolTips()
            self.drawWeights()
            self.drawIndexes()
            self.drawComponentKeywords()
        
        if replot:
            self.replot()
            
    def drawComponentKeywords(self):
        if self.showComponentAttribute == None:
            return
        
        if self.layout is None or self.graph is None or self.items is None:
            return
        
        if str(self.showComponentAttribute) not in self.items.domain:
            self.showComponentAttribute = None
            return
        
        components = Orange.network.nx.algorithms.components.connected_components(self.graph)
        
        for component in components:
            if len(component) == 0:
                continue
            
            vertices = [vertex for vertex in component if self.networkCurve.vertices[vertex].show]
    
            if len(vertices) == 0:
                continue
            
            xes = [self.networkCurve.coors[vertex][0] for vertex in vertices]  
            yes = [self.networkCurve.coors[vertex][1] for vertex in vertices]  
                                  
            x1 = sum(xes) / len(xes)
            y1 = sum(yes) / len(yes)
            
            lbl = str(self.items[component[0]][str(self.showComponentAttribute)])
            
            mkey = self.addMarker(lbl, float(x1), float(y1), alignment=Qt.AlignCenter, size=self.fontSize)
    
    def drawToolTips(self):
      # add ToolTips
      self.tooltipData = []
      self.tooltipKeys = {}
      self.tips.removeAll()
      if len(self.tooltipText) > 0:
        for vertex in self.networkCurve.vertices.itervalues():
          if not vertex.show:
            continue
          
          x1 = self.networkCurve.coors[vertex.index][0]
          y1 = self.networkCurve.coors[vertex.index][1]
          lbl = ""
          values = self.items[vertex.index]
          for ndx in self.tooltipText:
              if not ndx in self.items.domain:
                  continue
              
              value = str(values[ndx])
              # wrap text
              i = 0
              while i < len(value) / 100:
                  value = value[:((i + 1) * 100) + i] + "\n" + value[((i + 1) * 100) + i:]
                  i += 1
                  
              lbl = lbl + str(value) + "\n"
    
          if lbl != '':
            lbl = lbl[:-1]
            self.tips.addToolTip(x1, y1, lbl)
            self.tooltipKeys[vertex.index] = len(self.tips.texts) - 1
                   
    def drawLabels(self):
        if len(self.labelText) > 0:
            for vertex in self.networkCurve.vertices.itervalues():
                if not vertex.show:
                    continue
                
                if self.labelsOnMarkedOnly and not (vertex.marked):
                    continue
                                  
                x1 = self.networkCurve.coors[vertex.index][0]
                y1 = self.networkCurve.coors[vertex.index][1]
                lbl = ""
                values = self.items[vertex.index]
                if self.showMissingValues:
                    lbl = ", ".join([str(values[ndx]) for ndx in self.labelText])
                else:
                    lbl = ", ".join([str(values[ndx]) for ndx in self.labelText if str(values[ndx]) != '?'])
                #if not self.showMissingValues and lbl == '':
                #    continue 
                
                if lbl:
                    vertex.label = lbl
                    mkey = self.addMarker(lbl, float(x1), float(y1), alignment=Qt.AlignBottom, size=self.fontSize)
                    self.markerKeys[vertex.index] = mkey    
                     
    def drawIndexes(self):
        if self.showIndexes:
            for vertex in self.networkCurve.vertices.itervalues():
                if not vertex.show:
                    continue
                
                if self.labelsOnMarkedOnly and not (vertex.marked):
                    continue
                                  
                x1 = self.networkCurve.coors[vertex.index][0]
                y1 = self.networkCurve.coors[vertex.index][1]
    
                lbl = str(vertex.index)
                mkey = self.addMarker(lbl, float(x1), float(y1), alignment=Qt.AlignTop, size=self.fontSize)
                self.markerKeys['index ' + str(vertex.index)] = mkey         
    
    def drawWeights(self):
        if self.showWeights:
            for edge in self.edges:
                if not (edge.u.show and edge.v.show):
                    continue
                
                if self.labelsOnMarkedOnly and not (edge.u.marked and edge.v.marked):
                    continue
                                  
                x1 = (self.networkCurve.coors[edge.u.index][0] + self.networkCurve.coors[edge.v.index][0]) / 2
                y1 = (self.networkCurve.coors[edge.u.index][1] + self.networkCurve.coors[edge.v.index][1]) / 2
                
                if edge.weight == None:
                    lbl = "None"
                else:
                    lbl = "%.2f" % float(edge.weight)
                
                mkey = self.addMarker(lbl, float(x1), float(y1), alignment=Qt.AlignCenter, size=self.fontSize)
                self.markerKeys[(edge.u, edge.v)] = mkey
                            
    def getColorIndeces(self, table, attribute, palette):
        colorIndices = {}
        colorIndex = None
        minValue = None
        maxValue = None
        
        if attribute[0] != "(" or attribute[ -1] != ")":
            i = 0
            for var in table.domain.variables:
                if var.name == attribute:
                    colorIndex = i
                    if var.varType == orange.VarTypes.Discrete: 
                        colorIndices = getVariableValueIndices(var, colorIndex)
                        
                i += 1
            metas = table.domain.getmetas()
            for i, var in metas.iteritems():
                if var.name == attribute:
                    colorIndex = i
                    if var.varType == orange.VarTypes.Discrete: 
                        colorIndices = getVariableValueIndices(var, colorIndex)
    
        colorIndices['?'] = len(colorIndices)
        palette.setNumberOfColors(len(colorIndices))
        
        if colorIndex != None and table.domain[colorIndex].varType == orange.VarTypes.Continuous:
            minValue = float(min([x[colorIndex].value for x in table if x[colorIndex].value != "?"] or [0.0]))
            maxValue = float(max([x[colorIndex].value for x in table if x[colorIndex].value != "?"] or [0.0]))
            
        return colorIndices, colorIndex, minValue, maxValue
    
    def set_edge_color(self, attribute):
        if self.graph is None:
            return
        
        colorIndices, colorIndex, minValue, maxValue = self.getColorIndeces(self.links, attribute, self.discEdgePalette)
    
        for index in range(len(self.networkCurve.edges)):
            if colorIndex != None:
                links_index = self.networkCurve.edges[index].links_index
                if links_index == None:
                    continue
                
                if self.links.domain[colorIndex].varType == orange.VarTypes.Continuous:
                    newColor = self.discEdgePalette[0]
                    if str(self.links[links_index][colorIndex]) != "?":
                        if maxValue == minValue:
                            newColor = self.discEdgePalette[0]
                        else:
                            value = (float(self.links[links_index][colorIndex].value) - minValue) / (maxValue - minValue)
                            newColor = self.contEdgePalette[value]
                        
                    self.networkCurve.set_edge_color(index, newColor)
                    
                elif self.links.domain[colorIndex].varType == orange.VarTypes.Discrete:
                    newColor = self.discEdgePalette[colorIndices[self.links[links_index][colorIndex].value]]
                    if self.links[links_index][colorIndex].value == "0":
                      self.networkCurve.set_edge_color(index, newColor, nocolor=1)
                    else:
                      self.networkCurve.set_edge_color(index, newColor)
                    
            else:
                newColor = self.discEdgePalette[0]
                self.networkCurve.set_edge_color(index, newColor)
        
        self.replot()
    
    def set_node_color(self, attribute, nodes=None):
        if self.graph is None:
            return
        
        if nodes is None:
            nodes = self.networkCurve.vertices.itervalues()
        else:
            nodes = (self.networkCurve.vertices[nodes] for node in nodes) 
            
        colorIndices, colorIndex, minValue, maxValue = self.getColorIndeces(self.items, attribute, self.discPalette)
    
        if colorIndex is not None and self.items.domain[colorIndex].varType == orange.VarTypes.Continuous:
            for vertex in nodes:
                v = vertex.index
                newColor = self.discPalette[0]
                
                if str(self.items[v][colorIndex]) != "?":
                    if maxValue == minValue:
                        newColor = self.discPalette[0]
                    else:
                        value = (float(self.items[v][colorIndex].value) - minValue) / (maxValue - minValue)
                        newColor = self.contPalette[value]
                    
                self.networkCurve.set_node_color(v, newColor)
                
        elif colorIndex is not None and self.items.domain[colorIndex].varType == orange.VarTypes.Discrete:
            for vertex in nodes:
                v = vertex.index
                newColor = self.discPalette[colorIndices[self.items[v][colorIndex].value]]
                self.networkCurve.set_node_color(v, newColor)
        else:
            self.networkCurve.set_nodes_color(nodes, self.discPalette[0])
            
        self.replot()
        
    def setLabelText(self, attributes):
        self.labelText = []
        if self.layout is None or self.graph is None or self.items is None:
            return
        
        if isinstance(self.items, orange.ExampleTable):
            data = self.items
            for att in attributes:
                for i in range(len(data.domain)):
                    if data.domain[i].name == att:
                        self.labelText.append(i)
                        
                if self.items.domain.hasmeta(att):
                        self.labelText.append(self.items.domain.metaid(att))
    
    def setTooltipText(self, attributes):
        self.tooltipText = []
        if self.layout is None or self.graph is None or self.items is None:
            return
        
        if isinstance(self.items, orange.ExampleTable):
            data = self.items
            for att in attributes:
                for i in range(len(data.domain)):
                    if data.domain[i].name == att:
                        self.tooltipText.append(i)
                        
                if self.items.domain.hasmeta(att):
                        self.tooltipText.append(self.items.domain.metaid(att))
                        
    def setEdgeLabelText(self, attributes):
        self.edgeLabelText = []
        if self.layout is None or self.graph is None or self.items is None:
            return
        
    def change_graph(self, newgraph, inter_nodes, add_nodes, remove_nodes):
        self.graph = newgraph
        
        [self.networkCurve.vertices.pop(key) for key in remove_nodes]
        self.networkCurve.vertices.update(dict((v, NodeItem(v)) for v in add_nodes))
        vertices = self.networkCurve.vertices
        
        #build edge index
        row_ind = {}
        if self.links is not None and len(self.links) > 0:
          for i, r in enumerate(self.links):
              u = int(r['u'].value)
              v = int(r['v'].value)
              if u in self.graph and v in self.graph:
                  u_dict = row_ind.get(u, {})
                  v_dict = row_ind.get(v, {})
                  u_dict[v] = i
                  v_dict[u] = i
                  row_ind[u] = u_dict
                  row_ind[v] = v_dict
                  
        #add edges
        if self.links is not None and len(self.links) > 0:
            links = self.links
            links_indices = (row_ind[i + 1][j + 1] for (i, j) in self.graph.edges())
            labels = ([str(row[r].value) for r in range(2, len(row))] for row in (links[links_index] for links_index in links_indices))
            
            if self.graph.is_directed():
                edges = [EdgeItem(vertices[i], vertices[j],
                    self.graph[i][j].get('weight', 1), 0, 1, links_index, label) for \
                    ((i, j), links_index, label) in zip(self.graph.edges(), \
                                                        links_indices, labels)]
            else:
                edges = [EdgeItem(vertices[i], vertices[j],
                    self.graph[i][j].get('weight', 1), links_index, label) for \
                    ((i, j), links_index, label) in zip(self.graph.edges(), \
                                                        links_indices, labels)]
        elif self.graph.is_directed():
            edges = [EdgeItem(vertices[i], vertices[j],
                                      self.graph[i][j].get('weight', 1), 0, 1) for (i, j) in self.graph.edges()]
        else:
            edges = [EdgeItem(vertices[i], vertices[j],
                                      self.graph[i][j].get('weight', 1)) for (i, j) in self.graph.edges()]
                  
        #self.minEdgeWeight = min(edge.weight for edge in edges) if len(edges) > 0 else 0
        #self.maxEdgeWeight = max(edge.weight for edge in edges) if len(edges) > 0 else 0
        
        #if self.minEdgeWeight is None: 
        #    self.minEdgeWeight = 0 
        
        #if self.maxEdgeWeight is None: 
        #    self.maxEdgeWeight = 0 
                          
        #self.maxEdgeSize = 10
            
        #self.setEdgesSize()
        #self.setVerticesSize()
        
        self.networkCurve.coors = self.layout.map_to_graph(self.graph)
        self.networkCurve.edges = edges
        self.networkCurve.changed()
        
    def set_graph_layout(self, graph, layout, curve=None, items=None, links=None):
        self.clear()
        self.vertexDegree = []
        #self.vertices_old = {}
        #self.vertices = []
        #self.edges_old = {}
        #self.edges = []
        self.minEdgeWeight = sys.maxint
        self.maxEdgeWeight = 0
        
        if graph is None or layout is None:
            self.graph = None
            self.layout = None
            self.networkCurve = None
            self.items = None
            self.links = None
            xMin = self.axisScaleDiv(QwtPlot.xBottom).interval().minValue()
            xMax = self.axisScaleDiv(QwtPlot.xBottom).interval().maxValue()
            yMin = self.axisScaleDiv(QwtPlot.yLeft).interval().minValue()
            yMax = self.axisScaleDiv(QwtPlot.yLeft).interval().maxValue()
            self.addMarker("no network", (xMax - xMin) / 2, (yMax - yMin) / 2, alignment=Qt.AlignCenter, size=self.fontSize)
            self.tooltipNeighbours = 0
            self.replot()
            return
        
        self.graph = graph
        self.layout = layout
        self.networkCurve = NetworkCurve(self) if curve is None else curve
        self.items = items if items is not None else self.graph.items()
        self.links = links if links is not None else self.graph.links()
        
        #add nodes
        #self.vertices_old = [(None, []) for v in self.graph]
        vertices = dict((v, NodeItem(v)) for v in self.graph)
        
        #build edge index
        row_ind = {}
        if self.links is not None and len(self.links) > 0:
          for i, r in enumerate(self.links):
              u = int(r['u'].value)
              v = int(r['v'].value)
              if u in self.graph and v in self.graph:
                  u_dict = row_ind.get(u, {})
                  v_dict = row_ind.get(v, {})
                  u_dict[v] = i
                  v_dict[u] = i
                  row_ind[u] = u_dict
                  row_ind[v] = v_dict
              
        #add edges
        if self.links is not None and len(self.links) > 0:
            links = self.links
            links_indices = (row_ind[i + 1][j + 1] for (i, j) in self.graph.edges())
            labels = ([str(row[r].value) for r in range(2, len(row))] for row in (links[links_index] for links_index in links_indices))
            
            if self.graph.is_directed():
                edges = [EdgeItem(vertices[i], vertices[j],
                    graph[i][j].get('weight', 1), 0, 1, links_index, label) for \
                    ((i, j), links_index, label) in zip(self.graph.edges(), \
                                                        links_indices, labels)]
            else:
                edges = [EdgeItem(vertices[i], vertices[j],
                    graph[i][j].get('weight', 1), links_index, label) for \
                    ((i, j), links_index, label) in zip(self.graph.edges(), \
                                                        links_indices, labels)]
        elif self.graph.is_directed():
            edges = [EdgeItem(vertices[i], vertices[j],
                                      graph[i][j].get('weight', 1), 0, 1) for (i, j) in self.graph.edges()]
        else:
            edges = [EdgeItem(vertices[i], vertices[j],
                                      graph[i][j].get('weight', 1)) for (i, j) in self.graph.edges()]
        
        self.minEdgeWeight = min(edge.weight for edge in edges) if len(edges) > 0 else 0
        self.maxEdgeWeight = max(edge.weight for edge in edges) if len(edges) > 0 else 0
        
        if self.minEdgeWeight is None: 
            self.minEdgeWeight = 0 
        
        if self.maxEdgeWeight is None: 
            self.maxEdgeWeight = 0 
                          
        self.maxEdgeSize = 10
            
        self.setEdgesSize()
        self.setVerticesSize()
        
        self.networkCurve.coors = self.layout.map_to_graph(self.graph)
        self.networkCurve.vertices = vertices
        self.networkCurve.edges = edges
        self.networkCurve.changed()
        
    def setEdgesSize(self):
        if self.maxEdgeWeight > self.minEdgeWeight:
            #print 'maxEdgeSize',self.maxEdgeSize
            #print 'maxEdgeWeight',self.maxEdgeWeight
            #print 'minEdgeWeight',self.minEdgeWeight
            k = (self.maxEdgeSize - 1) / (self.maxEdgeWeight - self.minEdgeWeight)
            for edge in self.networkCurve.edges:
                if edge.weight == None:
                    size = 1
                    edge.pen = QPen(edge.pen.color(), size)
                    edge.pen.setCapStyle(Qt.RoundCap)
                else:
                    if self.invertEdgeSize:
                        size = (self.maxEdgeWeight - edge.weight - self.minEdgeWeight) * k + 1
                    else:
                        size = (edge.weight - self.minEdgeWeight) * k + 1
                    edge.pen = QPen(edge.pen.color(), size)
                    edge.pen.setCapStyle(Qt.RoundCap)
        else:
            for edge in self.networkCurve.edges:
                edge.pen = QPen(edge.pen.color(), 1)
                edge.pen.setCapStyle(Qt.RoundCap)
                
    def setVerticesSize(self, column=None, inverted=0):
        if self.layout is None or self.graph is None or self.items is None:
            return
        
        column = str(column)
        
        if column in self.items.domain or (column.startswith("num of ") and column.replace("num of ", "") in self.items.domain):
            values = []
            
            if column in self.items.domain:
                values = [self.items[x][column].value for x in self.graph if not self.items[x][column].isSpecial()]
            else:
                values = [len(self.items[x][column.replace("num of ", "")].value.split(',')) for x in self.graph]
          
            minVertexWeight = float(min(values or [0]))
            maxVertexWeight = float(max(values or [0]))
            
            if maxVertexWeight - minVertexWeight == 0:
                k = 1 #doesn't matter
            else:
                k = (self.maxVertexSize - self.minVertexSize) / (maxVertexWeight - minVertexWeight)
            
            def getValue(v):
                if v.isSpecial():
                    return minVertexWeight
                else:
                    return float(v)
                 
            if inverted:
                for key, vertex in self.networkCurve.vertices.iteritems():
                    if column in self.items.domain:
                        vertex.size = self.maxVertexSize - ((getValue(self.items[vertex.index][column]) - minVertexWeight) * k)
                    else:
                        vertex.size = self.maxVertexSize - ((len(self.items[vertex.index][column.replace("num of ", "")].value.split(',')) - minVertexWeight) * k)
                    
                    
                    vertex.pen.setWidthF(1 + float(vertex.size) / 20)
            else:
                for key, vertex in self.networkCurve.vertices.iteritems():
                    if column in self.items.domain:
                        vertex.size = (getValue(self.items[vertex.index][column]) - minVertexWeight) * k + self.minVertexSize
                    else:
                        vertex.size = (float(len(self.items[vertex.index][column.replace("num of ", "")].value.split(','))) - minVertexWeight) * k + self.minVertexSize
                        
                    #print vertex.size
                    vertex.pen.setWidthF(1 + float(vertex.size) / 20)
        else:
            for key, vertex in self.networkCurve.vertices.iteritems():
                vertex.size = self.maxVertexSize
                vertex.pen.setWidthF(1 + float(vertex.size) / 20)
      
    def updateCanvas(self):
        self.setAxisAutoScale(self.xBottom)
        self.setAxisAutoScale(self.yLeft)
        self.updateData()
        self.replot()  
    
    def zoomExtent(self):
        self.setAxisAutoScale(self.xBottom)
        self.setAxisAutoScale(self.yLeft)
        self.replot()
        
    def zoomSelection(self):
        selection = self.networkCurve.get_selected_nodes()
        if len(selection) > 0: 
            x = [self.networkCurve.coors[v][0] for v in selection]
            y = [self.networkCurve.coors[v][1] for v in selection]
    
            oldXMin = self.axisScaleDiv(QwtPlot.xBottom).interval().minValue()
            oldXMax = self.axisScaleDiv(QwtPlot.xBottom).interval().maxValue()
            oldYMin = self.axisScaleDiv(QwtPlot.yLeft).interval().minValue()
            oldYMax = self.axisScaleDiv(QwtPlot.yLeft).interval().maxValue()
            newXMin = min(x)
            newXMax = max(x)
            newYMin = min(y)
            newYMax = max(y)
            self.zoomStack.append((oldXMin, oldXMax, oldYMin, oldYMax))
            self.setAxisScale(QwtPlot.xBottom, newXMin - 100, newXMax + 100)
            self.setAxisScale(QwtPlot.yLeft, newYMin - 100, newYMax + 100)
            self.replot()
                    



"""
<name>Net File</name>
<description>Reads data from a graf file (Pajek networks (.net) files and GML network files).</description>
<icon>icons/NetworkFile.svg</icon>
<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact> 
<priority>6410</priority>
"""
import sys
import os.path
import user

import Orange
import OWGUI

from OWWidget import *

class OWNxFile(OWWidget):

    settingsList = ["recentFiles", "recentDataFiles", "recentEdgesFiles", "auto_table"]

    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, "Nx File", wantMainArea=False)

        self.inputs = []
        self.outputs = [("Network", Orange.network.Graph), ("Items", Orange.data.Table)]

        #set default settings
        self.recentFiles = ["(none)"]
        self.recentDataFiles = ["(none)"]
        self.recentEdgesFiles = ["(none)"]
        self.auto_table = True

        self.domain = None
        self.graph = None
        self.auto_items = None

        #get settings from the ini file, if they exist
        self.loadSettings()

        #GUI
        self.controlArea.layout().setMargin(4)
        self.box = OWGUI.widgetBox(self.controlArea, box="Graph File", orientation="vertical")
        hb = OWGUI.widgetBox(self.box, orientation="horizontal")
        self.filecombo = OWGUI.comboBox(hb, self, "filename")
        self.filecombo.setMinimumWidth(250)
        button = OWGUI.button(hb, self, '...', callback=self.browseNetFile, disabled=0)
        button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)
        OWGUI.checkBox(self.box, self, "auto_table", "Build graph data table automatically", callback=lambda: self.selectNetFile(self.filecombo.currentIndex()))

        self.databox = OWGUI.widgetBox(self.controlArea, box="Vertices Data File", orientation="horizontal")
        self.datacombo = OWGUI.comboBox(self.databox, self, "dataname")
        self.datacombo.setMinimumWidth(250)
        button = OWGUI.button(self.databox, self, '...', callback=self.browseDataFile, disabled=0)
        button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)

        self.edgesbox = OWGUI.widgetBox(self.controlArea, box="Edges Data File", orientation="horizontal")
        self.edgescombo = OWGUI.comboBox(self.edgesbox, self, "edgesname")
        self.edgescombo.setMinimumWidth(250)
        button = OWGUI.button(self.edgesbox, self, '...', callback=self.browseEdgesFile, disabled=0)
        button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        button.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)

        # info
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoa = OWGUI.widgetLabel(box, 'No data loaded.')
        self.infob = OWGUI.widgetLabel(box, ' ')
        self.infoc = OWGUI.widgetLabel(box, ' ')
        self.infod = OWGUI.widgetLabel(box, ' ')

        OWGUI.rubber(self.controlArea)
        self.resize(150, 100)
        self.activateLoadedSettings()

        # connecting GUI to code
        self.connect(self.filecombo, SIGNAL('activated(int)'), self.selectNetFile)
        self.connect(self.datacombo, SIGNAL('activated(int)'), self.selectDataFile)
        self.connect(self.edgescombo, SIGNAL('activated(int)'), self.selectEdgesFile)

    # set the comboboxes
    def setFileLists(self):
        self.filecombo.clear()
        if not self.recentFiles:
            self.filecombo.addItem("(none)")
        for file in self.recentFiles:
            if file == "(none)":
                self.filecombo.addItem("(none)")
            else:
                self.filecombo.addItem(os.path.split(file)[1])
        self.filecombo.addItem("Browse documentation networks...")

        self.datacombo.clear()
        if not self.recentDataFiles:
            self.datacombo.addItem("(none)")
        for file in self.recentDataFiles:
            if file == "(none)":
                self.datacombo.addItem("(none)")
            else:
                self.datacombo.addItem(os.path.split(file)[1])

        self.edgescombo.clear()
        if not self.recentEdgesFiles:
            self.edgescombo.addItem("(none)")
        for file in self.recentEdgesFiles:
            if file == "(none)":
                self.edgescombo.addItem("(none)")
            else:
                self.edgescombo.addItem(os.path.split(file)[1])

        self.filecombo.updateGeometry()
        self.datacombo.updateGeometry()
        self.edgescombo.updateGeometry()

    def activateLoadedSettings(self):
        # remove missing data set names
        self.recentFiles = filter(os.path.exists, self.recentFiles)
        self.recentDataFiles = filter(os.path.exists, self.recentDataFiles)
        self.recentEdgesFiles = filter(os.path.exists, self.recentEdgesFiles)

        self.recentFiles.append("(none)")
        self.recentDataFiles.append("(none)")
        self.recentEdgesFiles.append("(none)")
        self.setFileLists()

        if len(self.recentFiles) > 0 and os.path.exists(self.recentFiles[0]):
            self.selectNetFile(0)

        # if items not loaded with the network, load previous items
        if len(self.recentDataFiles) > 1 and \
            self.recentDataFiles[0] == 'none' and \
                os.path.exists(self.recentDataFiles[1]):
            self.selectDataFile(1)

        # if links not loaded with the network, load previous links    
        if len(self.recentEdgesFiles) > 1 and \
            self.recentEdgesFiles[0] == 'none' and \
                os.path.exists(self.recentEdgesFiles[1]):
            self.selectEdgesFile(1)

    # user selected a graph file from the combo box
    def selectNetFile(self, n):
        if n < len(self.recentFiles) :
            name = self.recentFiles[n]
            self.recentFiles.remove(name)
            self.recentFiles.insert(0, name)
        elif n:
            self.browseNetFile(1)

        if len(self.recentFiles) > 0:
            self.setFileLists()
            self.openFile(self.recentFiles[0])

    # user selected a data file from the combo box
    def selectEdgesFile(self, n):
        if n < len(self.recentEdgesFiles) :
            name = self.recentEdgesFiles[n]
            self.recentEdgesFiles.remove(name)
            self.recentEdgesFiles.insert(0, name)

        if len(self.recentEdgesFiles) > 0:
            self.setFileLists()
            self.addEdgesFile(self.recentEdgesFiles[0])

    def selectDataFile(self, n):
        if n < len(self.recentDataFiles) :
            name = self.recentDataFiles[n]
            self.recentDataFiles.remove(name)
            self.recentDataFiles.insert(0, name)

        if len(self.recentDataFiles) > 0:
            self.setFileLists()
            self.addDataFile(self.recentDataFiles[0])

    def readingFailed(self, infoa='No data loaded', infob='', infoc='', infod=''):
        self.graph = None
        self.send("Network", None)
        self.send("Items", None)
        self.infoa.setText(infoa)
        self.infob.setText(infob)
        self.infoc.setText(infoc)
        self.infod.setText(infod)

    def openFile(self, fn):
        """Read network from file."""

        # read network file
        if fn == "(none)":
            self.readingFailed()
            return

        fileExt = lower(os.path.splitext(fn)[1])
        if not fileExt in (".net", ".gml", ".gpickle"):
            self.readingFailed(infob='Network file type not supported')
            return

        #try:
        net = Orange.network.readwrite.read(fn, auto_table=self.auto_table)

        #except:
        #    self.readingFailed(infob='Could not read file')
        #    return

        if net == None:
            self.readingFailed(infob='Error reading file')
            return

        if self.auto_table:
            self.infoc.setText("Vertices data generated and added automatically")
            self.auto_items = net.items()
        else:
            self.auto_items = None

        self.infoa.setText("%d nodes" % net.number_of_nodes())

        if net.is_directed():
            self.infob.setText("Directed graph")
        else:
            self.infob.setText("Undirected graph")

        # make new data and send it
        fName = os.path.split(fn)[1]
        if "." in fName:
            #data.name = string.join(string.split(fName, '.')[:-1], '.')
            pass
        else:
            #data.name = fName
            pass

        self.graph = net

        # Find items data file for selected network
        items_candidate = os.path.splitext(fn)[0] + ".tab"
        if os.path.exists(items_candidate):
            self.readDataFile(items_candidate)
            if items_candidate in self.recentDataFiles:
                self.recentDataFiles.remove(items_candidate)
            self.recentDataFiles.insert(0, items_candidate)
        elif os.path.exists(os.path.splitext(fn)[0] + "_items.tab"):
            items_candidate = os.path.splitext(fn)[0] + "_items.tab"
            self.readDataFile(items_candidate)
            if items_candidate in self.recentDataFiles:
                self.recentDataFiles.remove(items_candidate)
            self.recentDataFiles.insert(0, items_candidate)
        else:
            if '(none)' in self.recentDataFiles:
                self.recentDataFiles.remove('(none)')
            self.recentDataFiles.insert(0, '(none)')

        # Find links data file for selected network
        links_candidate = os.path.splitext(fn)[0] + "_links.tab"
        if os.path.exists(links_candidate):
            self.readEdgesFile(links_candidate)
            self.recentEdgesFiles.insert(0, links_candidate)
        else:
            if '(none)' in self.recentEdgesFiles:
                self.recentEdgesFiles.remove('(none)')
            self.recentEdgesFiles.insert(0, '(none)')

        self.setFileLists()

        self.send("Network", self.graph)
        if self.graph != None and self.graph.items() != None:
            self.send("Items", self.graph.items())
        else:
            self.send("Items", None)

    def addDataFile(self, fn):
        if fn == "(none)" or self.graph == None:

            if self.auto_items is not None:
                self.infoc.setText("Vertices data generated and added automatically")
                self.graph.set_items(self.auto_items)
            else:
                self.infoc.setText("No vertices data file specified")
                self.graph.set_items(None)

            self.send("Network", self.graph)
            self.send("Items", self.graph.items())
            return

        self.readDataFile(fn)

        self.send("Network", self.graph)
        self.send("Items", self.graph.items())

    def readDataFile(self, fn):
        table = ExampleTable(fn)

        if len(table) != self.graph.number_of_nodes():
            self.infoc.setText("Vertices data length does not match number of vertices")

            if '(none)' in self.recentDataFiles:
                self.recentDataFiles.remove('(none)')

            self.recentDataFiles.insert(0, '(none)')
            self.setFileLists()
            return

        items = self.graph.items()
        if items is not None and \
                'x' in items.domain and \
                'y' in items.domain and \
                len(self.graph.items()) == len(table) and \
                'x' not in table.domain and 'y' not in table.domain:
            xvar = items.domain['x']
            yvar = items.domain['y']
            tmp = Orange.data.Table(Orange.data.Domain([xvar, yvar], False), items)
            table = Orange.data.Table([table, tmp])

        self.graph.set_items(table)
        self.infoc.setText("Vertices data file added")

    def addEdgesFile(self, fn):
        if fn == "(none)" or self.graph == None:
            self.infod.setText("No edges data file specified")
            #self.graph.setattr("links", None)
            self.send("Network", self.graph)
            self.send("Items", None)
            return

        self.readEdgesFile(fn)

        self.send("Network", self.graph)
        self.send("Items", self.graph.items())

    def readEdgesFile(self, fn):
        table = ExampleTable(fn)
        if self.graph.is_directed():
            nEdges = len(self.graph.getEdges())
        else:
            nEdges = len(self.graph.getEdges()) / 2

        if len(table) != nEdges:
            self.infod.setText("Edges data length does not match number of edges")

            if '(none)' in self.recentEdgesFiles:
                self.recentEdgesFiles.remove('(none)')

            self.recentEdgesFiles.insert(0, '(none)')
            self.setFileLists()
            return

        self.graph.set_links(table)
        self.infod.setText("Edges data file added")

    def browseNetFile(self, inDemos=0):
        """user pressed the '...' button to manually select a file to load"""

        "Display a FileDialog and select a file"
        if inDemos:
            import os
            try:
                import pkg_resources
                startfile = pkg_resources.load_entry_point("Orange-Network", "orange.data.io.search_paths", "network")().next()[1]
            except:
                startfile = ""

#            if not startfile or not os.path.exists(startfile):
#                try:
#                    import win32api, win32con
#                    t = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE, "SOFTWARE\\Python\\PythonCore\\%i.%i\\PythonPath\\Orange" % sys.version_info[:2], 0, win32con.KEY_READ)
#                    t = win32api.RegQueryValueEx(t, "")[0]
#                    startfile = t[:t.find("orange")] + "orange\\doc\\networks"
#                except:
#                    startfile = ""
#
#            if not startfile or not os.path.exists(startfile):
#                d = OWGUI.__file__
#                if d[-8:] == "OWGUI.py":
#                    startfile = d[:-22] + "doc/networks"
#                elif d[-9:] == "OWGUI.pyc":
#                    startfile = d[:-23] + "doc/networks"
#
#            if not startfile or not os.path.exists(startfile):
#                d = os.getcwd()
#                if d[-12:] == "OrangeCanvas":
#                    startfile = d[:-12] + "doc/networks"
#                else:
#                    if d[-1] not in ["/", "\\"]:
#                        d += "/"
#                    startfile = d + "doc/networks"

            if not os.path.exists(startfile):
                QMessageBox.information(None, "File", "Cannot find the directory with example networks", QMessageBox.Ok + QMessageBox.Default)
                return
        else:
            if len(self.recentFiles) == 0 or self.recentFiles[0] == "(none)":
                if sys.platform == "darwin":
                    startfile = user.home
                else:
                    startfile = "."
            else:
                startfile = self.recentFiles[0]

        filename = unicode(QFileDialog.getOpenFileName(self, 'Open a Network File', startfile, "All network files (*.gpickle *.net *.gml)\nNetworkX graph as Python pickle (*.gpickle)\nPajek files (*.net)\nGML files (*.gml)\nAll files (*.*)"))

        if filename == "": return
        if filename in self.recentFiles: self.recentFiles.remove(filename)
        self.recentFiles.insert(0, filename)
        self.setFileLists()
        self.selectNetFile(0)

    def browseDataFile(self, inDemos=0):
        if self.graph == None:
            return

        #Display a FileDialog and select a file
        if len(self.recentDataFiles) == 0 or self.recentDataFiles[0] == "(none)":
            if len(self.recentFiles) == 0 or self.recentFiles[0] == "(none)":
                if sys.platform == "darwin":
                    startfile = user.home
                else:
                    startfile = "."
            else:
                startfile = os.path.dirname(self.recentFiles[0])

        else:
            startfile = self.recentDataFiles[0]

        filename = unicode(QFileDialog.getOpenFileName(self, 'Open a Vertices Data File', startfile, 'Data files (*.tab)\nAll files(*.*)'))

        if filename == "": return
        if filename in self.recentDataFiles: self.recentDataFiles.remove(filename)
        self.recentDataFiles.insert(0, filename)
        self.setFileLists()
        self.addDataFile(self.recentDataFiles[0])

    def browseEdgesFile(self, inDemos=0):
        if self.graph == None:
            return

        #Display a FileDialog and select a file
        if len(self.recentEdgesFiles) == 0 or self.recentEdgesFiles[0] == "(none)":
            if len(self.recentFiles) == 0 or self.recentFiles[0] == "(none)":
                if sys.platform == "darwin":
                    startfile = user.home
                else:
                    startfile = "."
            else:
                startfile = os.path.dirname(self.recentFiles[0])

        else:
            startfile = self.recentEdgesFiles[0]

        filename = unicode(QFileDialog.getOpenFileName(self, 'Open a Edges Data File', startfile, 'Data files (*.tab)\nAll files(*.*)'))

        if filename == "": return
        if filename in self.recentEdgesFiles: self.recentEdgesFiles.remove(filename)
        self.recentEdgesFiles.insert(0, filename)
        self.setFileLists()
        self.addEdgesFile(self.recentEdgesFiles[0])

    def setInfo(self, info):
        for (i, s) in enumerate(info):
            self.info[i].setText(s)

    def sendReport(self):
        self.reportSettings("Network file",
                            [("File name", self.filecombo.currentText()),
                             ("Vertices", self.graph.number_of_nodes()),
                             hasattr(self.graph, "is_directed") and ("Directed", OWGUI.YesNo[self.graph.is_directed()])])
        self.reportSettings("Vertices meta data", [("File name", self.datacombo.currentText())])
        self.reportData(self.graph.items(), None)
        self.reportSettings("Edges meta data", [("File name", self.edgescombo.currentText())])
        self.reportData(self.graph.links(), None, None)

if __name__ == "__main__":
    a = QApplication(sys.argv)
    owf = OWNxFile()
    owf.activateLoadedSettings()
    owf.show()
    a.exec_()
    owf.saveSettings()



import Orange.network

from matplotlib import pyplot as plt

# vertices are placed randomly in NetworkOptimization constructor
network = Orange.network.NetworkOptimization()

# read network from file
net = Orange.network.Network.read("combination.net")

components = net.get_connected_components()
print "Connected components"
print components
print

distribution = net.get_degree_distribution()
print "Degree distribution"
print distribution
print

degrees = net.get_degrees()
print "Degrees"
print degrees
print

hubs = net.get_hubs(3)
print "Hubs"
print hubs
print

path = net.get_shortest_paths(0, 2)
print "Shortest path"
print path
print

distance = net.get_distance(0, 2)
print "Distance"
print distance
print

diameter = net.get_diameter()
print "Diameter"
print diameter
print

subnet = Orange.network.Network(net.get_sub_graph([0, 1, 2, 3]))
subNetOptimization = Orange.network.NetworkOptimization(subnet)
subNetOptimization.fruchterman_reingold(100, 1000)

# read all edges in subnetwork and plot a line
for u, v in subnet.getEdges():
    x1, y1 = subnet.coors[0][u], subnet.coors[1][u]
    x2, y2 = subnet.coors[0][v], subnet.coors[1][v]
    plt.plot([x1, x2], [y1, y2], 'b-')        
        
# read x and y coordinates to Python list
x = [coordinate for coordinate in subnet.coors[0]]
y = [coordinate for coordinate in subnet.coors[1]]

# plot vertices of subnetwork
plt.plot(x, y, 'ro')
plt.savefig("network-graph-analysis.png")
